package solatium.blockchain.persistence;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import solatium.blockchain.HashedBlock;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.MissingResourceException;

@Slf4j
public class PersistenceBlockChain {

    private final File blockChainFileDirectory;
    private final ObjectMapper mapper;

    public PersistenceBlockChain(File blockChainFileDirectory) {
        this.blockChainFileDirectory = blockChainFileDirectory;
        mapper = new ObjectMapper();
    }

    public void write(HashedBlock hashedBlock) {
        String fileName = hashedBlock.getHash() + ".json";
        File myFile = new File(blockChainFileDirectory, fileName);

        try (FileOutputStream fileOutputStream = new FileOutputStream(myFile);
             DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream)) {

            String hashedBlockJson = mapper.writeValueAsString(hashedBlock);
            dataOutputStream.write(hashedBlockJson.getBytes());

            log.debug("writing HashedBlock to JSON file: " + fileName);

            dataOutputStream.flush();
            fileOutputStream.flush();
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    public List<HashedBlock> read() {
        List<HashedBlock> hashedBlockList = new ArrayList<>();
        File[] files = blockChainFileDirectory.listFiles();

        for (int i = 0; i < files.length; i++) {
            try (FileReader fileReader = new FileReader(files[i]);
                 BufferedReader bufferedReader = new BufferedReader(fileReader)) {

                StringBuilder stringBuilder = new StringBuilder();
                String line = bufferedReader.readLine();

                while (line != null) {
                    stringBuilder.append(line);
                    stringBuilder.append(System.lineSeparator());
                    line = bufferedReader.readLine();
                }

                String json = stringBuilder.toString();

                HashedBlock hashedBlock = new ObjectMapper().readValue(json, HashedBlock.class);
                hashedBlockList.add(hashedBlock);
            } catch (IOException e) {
                throw new MissingResourceException("Error while reading File", PersistenceBlockChain.class.toString(), blockChainFileDirectory.getPath());
            }
        }

        return hashedBlockList;
    }
}
